package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.*;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.service.*;
import com.yunzhi.shigongli.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
    * 商铺 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "商铺")
@RestController
@RequestMapping("/")
public class TaShopController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaShopController.class);

    @Autowired
    public ITaShopService iTaShopService;


    @Autowired
    public ITaImageService shopImagesService;

    @Autowired
    public ITaTargetTypeService taTargetTypeService;

    @Autowired
    public ITaPvService iTaPvService;

    @Autowired
    public ITaResourceService taResourceService;


    @Autowired
    public ITaSaveService iTaSaveService;

    @Autowired
    public ITaLikeService iTaLikeService;

    @Autowired
    public ITaShopAccountService iTaShopAccountService;

    @Autowired
    public ITaMiniappQrcodeService iTaMiniappQrcodeService;

    @Autowired
    public ITaShopPackageService iTaShopPackageService;

    @Value("${sysBiz.miniapp.shopPage}")
    public String shopPage;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/taShop",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ShopListVO.class)
    public ResponseBean taShopList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @ApiParam("店铺名称") @RequestParam(value = "shopName", required = false) String shopName,
                                   @ApiParam("手机号") @RequestParam(value = "phone", required = false) String phone) throws Exception{

		    IPage<ShopListVO> pg = new Page<>(pageNum, pageSize);
            IPage<ShopListVO> result = iTaShopService.getPageListBy(pg, shopName, phone);
            return ResponseBean.success(result);
    }

    /**
     * 导出商铺列表
     * @return
     */
    @RequestMapping(value="/admin/taShop/export",method= RequestMethod.GET)
    @ApiOperation(value="导出商铺列表", notes = "导出商铺列表", httpMethod = "GET", response = ShopListVO.class)
    public void exportShopList(@ApiParam("店铺名称") @RequestParam(value = "shopName", required = false) String shopName,
                                       @ApiParam("手机号") @RequestParam(value = "phone", required = false) String phone,
                                       HttpServletResponse response) throws Exception{

        IPage<ShopListVO> result = iTaShopService.getPageListBy(null, shopName, phone);
        ExcelUtils.flush(response, ShopListVO.class, result.getRecords(), "商铺列表");
    }

    /**
     * 保存对象
     * @param taShop 实体对象
     * @return
     */
    @RequestMapping(value="/admin/taShop",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taShopAdd(@ApiParam("保存内容") @RequestBody TaShop taShop) throws Exception {

        if (null == taShop.getStatus()) {
            taShop.setStatus(Constants.STATUS_READY);
        }

        try {
            iTaShopService.createNew(taShop);
            shopImagesService.updateImages(Constants.TARGET_SHOP, taShop.getShopId(), taShop.getImageList());
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseBean.error(e.getMessage(), ResponseBean.ERROR_UNAVAILABLE);
        }

        return ResponseBean.success(taShop);
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/taShop/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taShopDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaShopService.removeLogicById(id)){
            taResourceService.removeBy(Constants.TARGET_SHOP, id);
            iTaShopPackageService.removeByShop(id);
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taShop 实体对象
     * @return
     */
    @RequestMapping(value="/admin/taShop/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taShopUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaShop taShop) throws Exception{

        if (iTaShopService.updateById(taShop)){
            shopImagesService.updateImages(Constants.TARGET_SHOP, taShop.getShopId(), taShop.getImageList());
            taTargetTypeService.updateTypes(Constants.TARGET_SHOP, taShop.getShopId(), taShop.getTypeList());

            ResourceFieldVO resourceVO = iTaShopService.copyProperties(taShop);
            TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
            BeanUtils.copyProperties(resourceVO, taResource);
            taResource.setTargetName(taShop.getShopName());

            taResourceService.updateById(taResource);
            return ResponseBean.success(iTaShopService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }


    /**
     * 商铺营业
     * @param id  实体ID
     * @return
     */
    @RequestMapping(value="/admin/taShop/{id}/on",method= RequestMethod.PUT)
    @ApiOperation(value="商铺营业", notes = "商铺营业", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean publish(@ApiParam("对象ID") @PathVariable String id) throws Exception {

        TaShop taShop = iTaShopService.getById(id);
        taShop.setStatus(Constants.STATUS_NORMAL);

        if (iTaShopService.updateById(taShop)){
            TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
            taResource.setStatus(taShop.getStatus());
            taResourceService.updateById(taResource);

            return ResponseBean.success(taShop);
        }else {
            return ResponseBean.error("发布失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 商铺关闭
     * @param id  实体ID
     * @return
     */
    @RequestMapping(value="/admin/taShop/{id}/off",method= RequestMethod.PUT)
    @ApiOperation(value="商铺关闭", notes = "商铺关闭", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean unPublish(@ApiParam("对象ID") @PathVariable String id) throws Exception {

        TaShop taShop = iTaShopService.getById(id);
        taShop.setStatus(Constants.STATUS_READY);

        if (iTaShopService.updateById(taShop)){
            TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
            taResource.setStatus(taShop.getStatus());
            taResourceService.updateById(taResource);

            return ResponseBean.success(taShop);
        }else {
            return ResponseBean.error("取消发布失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }



    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/taShop/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taShopGet(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaShop taShop = iTaShopService.getById(id);
        if (null == taShop || Constants.STATUS_DELETED == taShop.getStatus()) {
            return ResponseBean.error("当前商铺信息不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
        taShop.setPvNum(taResource.getPvNum());
        taShop.setUvNum(taResource.getUvNum());
        taShop.setLikeNum(taResource.getLikeNum());
        taShop.setImageList(shopImagesService.getListBy(Constants.TARGET_SHOP, id));
        taShop.setTypeList(taTargetTypeService.getListBy(Constants.TARGET_SHOP, id));

        return ResponseBean.success(taShop);
    }

    @PostMapping("/admin/taShop/{id}/qrcode")
    @ApiOperation(value="店铺二维码", notes = "店铺二维码", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean generateQrCode(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaShop taShop = iTaShopService.getById(id);
        if (null == taShop || Constants.STATUS_DELETED == taShop.getStatus()) {
            return ResponseBean.error("当前商铺信息不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        QrCodeParamsVO qrCodeParamsVO = new QrCodeParamsVO();
        qrCodeParamsVO.setPage(shopPage);
        qrCodeParamsVO.setType(Constants.TARGET_SHOP);
        qrCodeParamsVO.setScene(new HashMap<String, Object>(){{
            put("id", id);
        }});
        String qrCode = iTaMiniappQrcodeService.createQrCode(qrCodeParamsVO);

        return ResponseBean.success(new HashMap<String, Object>(){{
            put("qrCode", qrCode);
        }});
    }

    /**
     * 微信端详情
     * @param id
     * @return
     * @throws Exception
     */
    @ApiOperation(value="微信端详情", notes = "微信端详情", httpMethod = "GET", response = ResponseBean.class)
    @GetMapping("/wx/shop/{id}")
    public ResponseBean getWxDetail(@ApiParam("对象ID") @PathVariable String id,
                                    @ApiParam("定位") @RequestParam("location") String location) throws Exception {
        TaPerson taPerson = getCurrentPerson();

        TaShop taShop = iTaShopService.getProfileBy(id, taPerson, location);
        if (Constants.STATUS_DELETED == taShop.getStatus()) {
            return ResponseBean.error("当前商铺信息不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        taShop.setImageList(shopImagesService.getListBy(Constants.TARGET_SHOP, id));

        // 统计 PV, UV
        try {
            taResourceService.recordPerson(Constants.TARGET_SHOP, id, getCurrentPerson());
        } catch (Exception e) {
            e.printStackTrace();
        }

        return ResponseBean.success(taShop);
    }

    /**
     * 商铺小程序首页
     */
    @RequestMapping(value="/wx/shop/manage/index",method= RequestMethod.GET)
    @ApiOperation(value="商铺小程序首页", notes = "商铺小程序首页", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getWxManageIndex() throws Exception{
        TaPerson taPerson = getCurrentPerson();
        if (!Constants.TARGET_SHOP.equals(taPerson.getPersonRole())) {
            return ResponseBean.error("请先进行人员身份绑定", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 获取所有的商铺信息
        List<TaShop> shopList = iTaShopService.getListByPerson(taPerson);
        if (null == shopList || shopList.size() < 1) {
            return ResponseBean.error("数据异常, 未找到关联商铺", ResponseBean.ERROR_UNAVAILABLE);
        }

        Map<String, Object> result = new HashMap<>();
        result.put("shopList", shopList);
        result.put("shop", shopList.get(0));

        return ResponseBean.success(result);
    }


    /**
     * 商铺订单列表
     * @param pageNum
     * @param pageSize
     * @param shopName
     * @param description
     * @param start
     * @param end
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/taShop/orderSub",method= RequestMethod.GET)
    @ApiOperation(value="商铺订单列表", notes = "商铺订单列表", httpMethod = "GET", response = ShopOrderVO.class)
    public ResponseBean getShopOrderList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                         @ApiParam("店铺名称") @RequestParam(value = "shopName", required = false) String shopName,
                                         @ApiParam("套餐名称") @RequestParam(value = "description", required = false) String description,
                                         @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                                         @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end) throws Exception {
        IPage<ShopOrderVO> pg = new Page<>(pageNum, pageSize);

        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<ShopOrderVO> result = iTaShopService.getShopOrderList(pg, shopName, description, startDate, endDate);

        return ResponseBean.success(result);
    }


    /**
     * 商铺订单列表导出
     * @param shopName
     * @param description
     * @param start
     * @param end
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/taShop/orderSub/export",method= RequestMethod.GET)
    @ApiOperation(value="商铺订单列表导出", notes = "商铺订单列表导出", httpMethod = "GET", response = ShopOrderVO.class)
    public void exportShopOrderList(@ApiParam("店铺名称") @RequestParam(value = "shopName", required = false) String shopName,
                                    @ApiParam("套餐名称") @RequestParam(value = "description", required = false) String description,
                                    @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                                    @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end,
                                    HttpServletResponse response) throws Exception {
        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<ShopOrderVO> result = iTaShopService.getShopOrderList(null, shopName, description, startDate, endDate);

        ExcelUtils.flush(response, ShopOrderVO.class, result.getRecords(), "商家统计");
    }


    /**
     * 商铺推广列表
     * @param pageNum
     * @param pageSize
     * @param shopName
     * @param description
     * @param start
     * @param end
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/taShop/commission",method= RequestMethod.GET)
    @ApiOperation(value="商铺推广列表", notes = "商铺推广列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getShopCommissionList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                         @ApiParam("店铺名称") @RequestParam(value = "shopName", required = false) String shopName,
                                         @ApiParam("套餐名称") @RequestParam(value = "description", required = false) String description,
                                         @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                                         @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end) throws Exception {
        IPage<ShopCommissionVO> pg = new Page<>(pageNum, pageSize);

        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<ShopCommissionVO> result = iTaShopService.getCommissionList(pg, shopName, description, startDate, endDate);

        return ResponseBean.success(result);
    }



    /**
     * 商铺推广列表
     * @param shopName
     * @param description
     * @param start
     * @param end
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/taShop/commission/export",method= RequestMethod.GET)
    @ApiOperation(value="商铺推广列表", notes = "商铺推广列表", httpMethod = "GET", response = ResponseBean.class)
    public void exportShopCommissionList(@ApiParam("店铺名称") @RequestParam(value = "shopName", required = false) String shopName,
                                         @ApiParam("套餐名称") @RequestParam(value = "description", required = false) String description,
                                         @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                                         @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end,
                                         HttpServletResponse response) throws Exception {

        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<ShopCommissionVO> result = iTaShopService.getCommissionList(null, shopName, description, startDate, endDate);

        ExcelUtils.flush(response, ShopCommissionVO.class, result.getRecords(), "商铺推广收入");
    }
}
